/*********************************************************\
*  AVR MegaSquirt - 2003                                  *
*  (C) 2003 [Michael Kristensen, mik@caffrey.dk]          *
***********************************************************
*  -- History --                                          *
*    release 0.1 - 06-03-2003                             *
*            http://caffrey.dk/megasquirt                 *
\*********************************************************/

#ifndef IAC_H
#define IAC_H

#include "softpwm.h"

extern struct iac_t iac;

//#define SIMPLE_IAC // this requires changes in config_t
#define PID_IAC

#ifdef SIMPLE_IAC
// defines for config.iac[]
#define RESYNC            0   // set to 0 to disable resync. Any other number is in 'FINE_PERIOD' resolution
#define COARSE_DEV        1   // [rpm/10], ex: 23 <=> 230 rpm deviation
#define ADJUST_COARSE     2   // number of steps moved at each adjustment, packed:
                              // Upper four bits for increasing of rpm, lower 4 bits for decreasing rpm
#define COARSE_PERIOD     3   // 1: 100ms, 2: 200ms, 3: 300ms, etc.
#define FINE_DEV          4   // [rpm/10], ex: 23 <=> 230 rpm deviation
#define ADJUST_FINE       5   // number of steps moved at each adjustment, packed:
                              // Upper four bits for increasing of rpm, lower 4 bits for decreasing rpm
#define FINE_PERIOD       6   // 1: 100ms, 2: 200ms, 3: 300ms, etc.

/*
Suggested configuration:
step_seq : depends on how the stepper is wired up, read the docs
conf : no power off, speed depends on the stepper
max_steps : 100
resync : 15 (every 15 seconds)
tps_thres : depends on throttle pot
fast_idle_temp : 60C degress
cold_rpm : 100 (1000 rpm)
warm_rpm : 85 (850 rpm)
cold_start_pos : 50
warm_start_pos : 20
decel_open : 0
coarse_dev : 20 (+/- 200 rpm)
adjust_coarse : 0x43
coarse_period : 1
fine_dev : 3 (+/- 30 rpm)
adjust_fine : 0x21
fine_period : 10 (every second)
*/

#endif



/*
#ifdef PID_IAC
// defines for config.iac[]
#define START_RPM         0   // right after cranking, this (rpm/10) is added to target_rpm such that the idle rpm temporarily is raised
#define START_COUNT       1   // the extra START_RPM is decreased in START_COUNT [sec/10]
#define PID_CONF          2   // parameters for the pid controller (asymmetric)
#define KP                3   // proportional multiplier
#define KI                4   // integral multiplier
#define KD                5   // differential multiplier
#define DEADBAND          6   // no movement of iac when |current-target_rpm| < deadband. [rpm/10]
#define INTEGRAL_LIMIT    7   // limit the integrator
#define TAU_INCREASE_RPM  8   // timedelay increasing rpm when engine is spinning too slow (keep this small)
#define TAU_DECREASE_RPM  9   // timedelay decreasing rpm when engine is spinning too fast

#endif
*/


/***********************************************************************/
// config.iac_conf bits:
#define power_off_iac  		0    // power off stepper 1 step cycle after each move
#define tps_follow  		1    // iac follows tps when above iac_tps_threshold
#define stepper_iac		2    // enable stepper based iac
#define precise_idle		3    // 0: on/off 1: pwm/stepper based
#define advance_control 	4   // 0: off 1: enabled
#define dualpwm 		5   // 0: off 1: enabled
#define iac_asymetric		6	// 1: asymetric (strong actuation below target)
#define iac_ignsoftidle		7	// 1: ignition-cut based soft-idle
/***********************************************************************/
// config_ext.iac_conf2 bits:
#define stepper_negative	0    // 0-old positive, 1-negative
#define toyota_stepper		1	// 1-toyota stepper at p259_4-7
#define float_stepper_pos	2	// 0-old, 1-float pos


//dont forget about these in global.h
// #define config_iac_pid_asymetric() (config.iac_conf & _BV(6))
// #define config_iac_ignsoftidle() (config.iac_conf & _BV(7))

#define iac_dualpwm_configured() (config.iac_conf & _BV( dualpwm ))
#define iac_tps_follow_configured() (config.iac_conf & _BV( tps_follow ))

#define iac_precise_idle_configured() (config.iac_conf & _BV(precise_idle))
// stepper only actuated if precise_idle is also true !
#define iac_stepper_configured() (config.iac_conf & _BV(stepper_iac))
// note that solenoid can be ON/OFF or precise_idle (PWM)
#define iac_solenoid_configured() ((config.iac_conf & _BV(stepper_iac)) == 0)
#define iac_pwm_configured() ((iac_precise_idle_configured()) && (iac_solenoid_configured()))

// use 0 bitvalue for special function (when using v2 IDL output for WOT switch) 
// Do we want independent bit to disable the stepper? grep stepperdisable
// It does not hurt to have stepper and softpwm output at the same time

// iac status bit defines:
#define busy         0
#define direction    1  // 1 == open, 0 == close
#define sync         2  // moving to fully closed position
#define no_integral  3  // no integral control!

// these variables are only used for stepper IAC (not for PWM-IAC)
struct iac_stepper_t {
	// uint8_t resync;	// for overclose
	uint8_t count;	// how many steps are requested
	uint8_t phase;	// needed (?) for actuation
};

struct iac_t {
  uint8_t status;        // uses the defines above

// For solenoid, iac.position != duty (duty is higher at low VBATT)
// duty (that also considers VBATT) is in idle_pwm_soft.designated_ratio
	uint8_t position;      // current position

// integral is the only "static" variable, the others are just used for lcd
	int16_t integral;
	// int16_t error;
	int16_t rpmdot;

// intermediate results, used for lcd
	int16_t pid;
	int16_t pid_p;
	int16_t pid_i;
	int16_t pid_d;
	
	uint8_t duty;

union {
#ifndef TRUE_PWM_IAC // new experimental
	struct softpwm_t idle_pwm_soft; // only used for PWM-IAC
#endif
	struct iac_stepper_t stepperonly;
} shared;

	int16_t adv_adjust;
	uint8_t target_rpm; // iac current target RPM [10 RPM]. Approaches the final-target exponentially
	uint8_t afterstart_duration;	// 0.1 sec timer to control extra air
	uint8_t afterstart_rpm;		// [10 RPM]
	uint16_t last_rpm;	// used by rpmdot [1 RPM]

//counter
	uint16_t closing_delay;
//last used PID-calculated position
	uint8_t last_pos;
};

// the number of steps besides config.iac[MAX_STEPS], to close the iac at powerup
#define IAC_SYNC 10
#define INTEGRAL_LIMIT_SCALE 7

void calc_iac_pos(void);
void init_iac(void);
void set_iac_pos(uint8_t pos);
uint8_t next_step(uint8_t phase);
void move_iac(void);
// void iac_pwm_soft(void);
// void fast_idle_valve(void);
void idle_solenoid(void);
uint16_t idle_advance_adjust(uint16_t adv);
uint8_t lookup_iac_clt_ref_pos( void );

#ifdef SOFTIDLE
void calc_idle_sparkcut(void);
#endif

#endif
